@@ -1,4 +1,4 @@
-# $Id: Mpp.pm,v 1.17 2012/06/09 20:36:20 pfeiffer Exp $
+# $Id: Mpp.pm,v 1.20 2013/01/31 22:06:36 pfeiffer Exp $
 
 =head1 NAME
 
@@ -68,10 +68,6 @@ our $error_found = 0;		# Non-zero if we found an error.  This is used
 				# to stop cleanly when we are doing a parallel
 				# build (unless -k is specified).
 our $failed_count = 0;		# How many targets failed.  TODO: my when moving build et al here
-our $build_cache_hits = 0;	# Number of the files changed that were
-				# imported from a build cache.
-our $rep_hits = 0;		# Number of the files changed that were
-				# imported from a rep.
 our $print_directory = 1;	# Default to printing it.
 our $log_level = 2;		# Default to logging. 1: STDOUT, 2: $logfile
 sub log($@);
@@ -124,10 +120,10 @@ sub log($@);
   @pending_signals{@signals_to_handle} = ( 0 ) x @signals_to_handle;
 
   END {
-    Mpp::log N_REP_HITS => $rep_hits
-      if $log_level && $rep_hits;
-    Mpp::log N_CACHE_HITS => $build_cache_hits
-      if $log_level && $build_cache_hits;
+    Mpp::log N_REP_HITS => $Mpp::Repository::hits
+      if $log_level && $Mpp::Repository::hits;
+    Mpp::log N_CACHE_HITS => $Mpp::BuildCache::hits
+      if $log_level && $Mpp::BuildCache::hits;
     Mpp::log N_FILES => $n_files_changed, $n_phony_targets_built, $failed_count
       if defined $logfh;		    # Don't create log for --help or --version.
     if( exists $Devel::DProf::{VERSION} ) { # Running with profiler?
@@ -247,7 +243,7 @@ sub log($@) {
       }
     }
     push @close_fhs, $logfh;
-    printf $logfh "3\01%s\nVERSION\01%s\01%vd\01%s\01\n", $invocation, $Mpp::VERSION, $^V, ARCHITECTURE;
+    printf $logfh "3\01%s\nVERSION\01%s\01%vd\01%s\01\n", $invocation, $Mpp::Text::VERSION, $^V, ARCHITECTURE;
 
     # If we're running with --traditional-recursive-make, then print the directory
     # when we're entering and exiting the program, because we may be running as
@@ -359,14 +355,19 @@ sub print_error {
 
 
 sub perform(@) {
-  #my @handles = @_;		# Arguments passed to wait_for.
+  #my @targets = @_;		# Arguments passed to wait_for.
   my $status;
   my $start_pid = $$;
-  my $error_message = $@ || '';
+  print $logfh "\f\n" if $Mpp::loop && &maybe_stop && defined $logfh;
+  my @handles = grep {
+    exists $_->{DONT_BUILD} or undef $_->{DONT_BUILD};
+    Mpp::build $_;			# Try to build the file, return handle if necessary.
+  } @_;
+  my $error_message = $@;
   unless( $error_message ) {
-    eval { $status = &wait_for }; # Wait for args to be built.
+    eval { $status = wait_for( @handles ) }; # Wait for args to be built.
 				# Wait for all the children to complete.
-    $error_message .= $@ if $@;	# Record any new error messages.
+    $error_message = $@ if $@;	# Record any new error messages.
   }
   {
     my $orig = '';
@@ -403,21 +404,61 @@ signal propagation.$orig Stopped}
     }
   }
 
-  if( $quiet_flag ) {
-    # Suppress following chatter.
-  } elsif( $n_files_changed || $rep_hits || $build_cache_hits || $n_phony_targets_built || $failed_count ) {
-    print "$progname: $n_files_changed file" . ($n_files_changed == 1 ? '' : 's') . ' updated' .
-      ($rep_hits ? ", $rep_hits repository import" . ($rep_hits == 1 ? '' : 's') : '') .
-      ($build_cache_hits ? ", $build_cache_hits build cache import" . ($build_cache_hits == 1 ? '' : 's') : '') .
-      ($error_found ? ',' : ' and') .
-      " $n_phony_targets_built phony target" . ($n_phony_targets_built == 1 ? '' : 's') . ' built' .
-      ($failed_count ? " and $failed_count target" . ($failed_count == 1 ? '' : 's') . " failed\n" : "\n");
-  } elsif( !$error_message ) {
-    print "$progname: no update necessary\n";
+  unless( $quiet_flag ) {
+    my $found;
+    my $msg = join '', map {
+      $found ||= $_->[1];
+      $_->[1] || !$_->[2] ?
+	sprintf( $_->[0], $_->[1], $_->[1] == 1 ? '' : 's' ) : # plural?
+	'';
+    } ["%d file%s updated", $n_files_changed],
+      [', %d repository import%s', $Mpp::Repository::hits, 1],
+      [', %d build cache import%s', $Mpp::BuildCache::hits, 1],
+      [($failed_count ? ',' : ' and') . ' %d phony target%s built', $n_phony_targets_built],
+      [" and %d target%s failed", $failed_count, 1];
+    if( $found ) {
+      print "$progname: $msg\n";
+    } elsif( !$error_message ) {
+      print "$progname: no update necessary\n";
+    }
   }
 
   print "$progname: Ending \@" . &$hires_time() . "\n" if $profile;
   print_error $error_message if $error_message;
+
+  if( $Mpp::loop ) {
+    &Mpp::File::update_build_infos;
+    if( defined $logfh ) {
+      Mpp::log N_REP_HITS => $Mpp::Repository::hits
+	if $log_level && $Mpp::Repository::hits;
+      Mpp::log N_CACHE_HITS => $Mpp::BuildCache::hits
+	if $log_level && $Mpp::BuildCache::hits;
+      Mpp::log N_FILES => $n_files_changed, $n_phony_targets_built, $failed_count;
+      ;
+    }
+    $n_files_changed = $Mpp::Repository::hits = $Mpp::BuildCache::hits = $n_phony_targets_built =
+      $failed_count = 0;
+    $Mpp::stop = 1;
+    my @dirs = $Mpp::File::root;
+    Mpp::File::touched_filesystem;
+    while( @dirs ) {
+      my $dinfo = pop @dirs;
+      next if exists $dinfo->{xREPOSITORY};
+      for( values %{$dinfo->{DIRCONTENTS}} ) {
+	if( $_->{DIRCONTENTS} ) {
+	  push @dirs, $_ unless Mpp::File::is_symbolic_link( $_ );
+                                # Don't traverse it if it's a symbolic link,
+                                # because then we'll find the same files twice.
+	} else {
+	  Mpp::File::may_have_changed( $_ );
+	  delete $_->{BUILD_HANDLE}; # Also forget we may have already built it.
+	}
+      }
+    }
+
+    goto &perform;		# Start over w/o loading makefiles or repositories again.
+  }
+
   exit 1 if $error_message || $status || !MAKEPP && $failed_count;
 				# 2004_12_06_scancache has a use case for not failing despite $failed_count
   exit 0;
